const GITHUB_API_URL = "https://api.github.com";
const RAWGIT_API_URL = "https://rawgit.com";

class Repository {
    /**
     * Create a repository instance for further purposes
     * @param {string} user 
     * @param {string} repositoryName 
     * @param {string?} hash 
     * @param {string?=master} branch 
     */
    constructor(user, repositoryName, hash, branch = "master") {
        this.user = user;
        this.repositoryName = repositoryName;
        this.hash = hash;
        this.branch = branch;
    }

    /**
     * Generate User/Repository slug for URL accessors
     * @return {string}
     */
    get slug() {
        return `${this.user}/${this.repositoryName}`;
    }

    /**
     * Genereate a CDN path to a certain file
     * @param {string} file 
     */
    generateCDNPath(file) {
        if (!this.hash) {
            console.log([
                `Hash property is missing in configuration, try calling`,
                `${GITHUB_API_URL}/repos/${this.user}/${this.repositoryName}/commits/master`,
                `to identify your sha hash for CDN generation`
            ]);

            return "";
        }

        return `${RAWGIT_API_URL}/${this.slug}/${this.hash}/${file}`;
    }

    /**
     * Genereate a pure content path (raw.githubusercontent.com) to a certain file
     * @param {string} file 
     */
    generateContentPath(file) {
        if (!this.branch) {
            console.log([
                `Branch identification is missing, you must pass a branch property`,
                `to deliver content directly from rawgit.com`
            ]);

            return "";
        }

        return `${RAWGIT_API_URL}/${this.slug}/${this.branch}/${file}`;
    }
}

exports = module.exports = ({ user, repository, hash, branch = "master" }) => {
    const repo = new Repository(user, repository, branch, hash);

    return {
        url: file => repo.generateContentPath(file),
        cdn: file => repo.generateCDNPath(file)
    };
};
